#include "Keyboard.h"
#include <Keyboard_de_DE.h>
const bool debug = 1;    // 1 for serial messages
const int debounce = 10; // how many times a value has to occur to trigger a keypress and not just be a bounce

const int swtchNo = 46; // number of interface elements that trigger once on press
const int btnNo = 3;   // number of interface elements that trigger continuously on press
const int chngNo = 12;  // number of interface elements that trigger when changed

const String swtchName[swtchNo] = { "GRP1",   "GRP2",   "GRP3",   "GRP4",   "GRP5",   "GRP6",   "ALARM",     "EJECT",      "SALVO",      "OVERRIDE", "LNAV", "RNAV", "REVDIR", "IMGMAG", "SWCAM", "TORSOLEGS", "LEGSTORSO", "NEARFR", "RFR", "LFOR", "RFOE", "LFR", "RTCL", "NEARFOE", "OBJECTIVES", "PAUSE", "F1",   "F2",   "F3",   "F4",   "F5",   "F6",   "F7",   "F8",   "F9",   "F10",   "F1O",  "F2O",  "F3O",  "F4O",  "F5O",  "F6O",  "F7O",  "F8O",  "F9O",  "F10O"  }; // name
const char swtchChar[swtchNo] =   { KEY_KP_1, KEY_KP_2, KEY_KP_3, KEY_KP_4, KEY_KP_5, KEY_KP_6, KEY_KP_PLUS, KEY_KP_MINUS, KEY_KP_SLASH, 'o',        'b',    'n',    'u',      'i',      'c',     't',         'z',         '1',      '3',   '6',    '7',    '2',   '4',    '5',       KEY_ACUTE,    KEY_ESC, KEY_F1, KEY_F2, KEY_F3, KEY_F4, KEY_F5, KEY_F6, KEY_F7, KEY_F8, KEY_F9, KEY_F10, KEY_F1, KEY_F2, KEY_F3, KEY_F4, KEY_F5, KEY_F6, KEY_F7, KEY_F8, KEY_F9, KEY_F10 }; // sent character
const int swtch[swtchNo] =        {  4,       3,        2,        7,        6,        5,        12,          11,           10,            50,         49,     47,     38,       36,       34,      30,          28,          37,       35,    33,     31,     29,    27,     25,       0,            1,       69,     68,     67,     66,     65,     64,     63,     62,     61,     60,      59,     58,     57,     56,     55,     54,     21,     20,     19,     18      }; // pin
const bool inv[swtchNo] =         {  0,       0,        0,        0,        0,        0,        0,           0,            0,             0,          0,      0,      0,        0,        0,       0,           0,           0,        0,     0,      0,      0,     0,      0,        0,            0,       0,      0,      0,      0,      0,      0,      0,      0,      0,      0,       0,      0,      0,      0,      0,      0,      0,      0,      0,      0       }; // inverted?

const String btnName[btnNo] =     { "COOL", "LOOKBACK", "LOOKDOWN" };
const char btnChar[btnNo] =       { 'f',    'y',        'x',       };
const int btn[btnNo] =            {  48,     40,         32        };

const String chngName[chngNo] =   { "AMS", "ARM", "AUTOCTR", "MNSHT", "CRCH", "HEATVIS", "RADMAP", "LIGHTAMP",   "ACTPAS", "LIGHT", "RADRNG",     "ECMOFF"     };
const char chngChar[chngNo] =     { '#',   '-',   'v',       '0',     'h',    'j',       'r',      KEY_O_UMLAUT, 'p',      'l',     KEY_A_UMLAUT, KEY_U_UMLAUT };
const int chng[chngNo] =          {  8,     9,     53,        52,      51,     46,        45,      44,            43,       42,     41,           39           };
 
int swtchDebounce[swtchNo];
int btnDebounce[btnNo];
int chngDebounce[chngNo];
bool swtchPrev[swtchNo];
bool btnPrev[btnNo];
bool chngPrev[chngNo];

void setup() {
  for (int i = 0; i < swtchNo; i++) { // Initialize all arrays
    swtchDebounce[i] = 0;
    swtchPrev[i] = 1;
  }
  for (int i = 0; i < btnNo; i++) {
    btnDebounce[i] = 0;
    btnPrev[i] = 1;
  }
  for (int i = 0; i < chngNo; i++) {
    chngDebounce[i] = 0;
    chngPrev[i] = 1;
  }
  
  if(debug) Serial.begin(9600);
  Keyboard.begin(KeyboardLayout_de_DE);

  for (int i = 0; i < swtchNo; i++) { 
    pinMode(swtch[i], INPUT_PULLUP);
  }
  for (int i = 0; i < btnNo; i++) {
    pinMode(btn[i], INPUT_PULLUP);
  }
  for (int i = 0; i < chngNo; i++) {
    pinMode(chng[i], INPUT_PULLUP);
  }

}

void loop() {

  for (int i = 0; i < swtchNo; i++) {
    bool inputState = digitalRead(swtch[i]);
    if(inv[swtchNo] == 1){
      inputState = !inputState; 
    }
    if (inputState != swtchPrev[i]) {
      swtchDebounce[i]++;
    }
    if (swtchDebounce[i] >= debounce) {
      if (inputState == LOW) {
        if(debug) Serial.println(swtchName[i]);
        Keyboard.print(swtchChar[i]);
      }
      swtchPrev[i] = !swtchPrev[i];
      swtchDebounce[i] = 0;
    }
  }

  for (int i = 0; i < btnNo; i++) {
    bool inputState = digitalRead(btn[i]);
    if (inputState != btnPrev[i]) {
      btnDebounce[i]++;
    }
    if (btnDebounce[i] >= debounce) {
      if (inputState == LOW) {
        if(debug) Serial.print(btnName[i]);
        if(debug) Serial.println(" - PRESSED");
        Keyboard.press(btnChar[i]);
      } else {
        if(debug) Serial.print(btnName[i]);
        if(debug) Serial.println(" - RELEASED");
        Keyboard.release(btnChar[i]);
      }
      btnPrev[i] = !btnPrev[i];
      btnDebounce[i] = 0;
    }
  }

  for (int i = 0; i < chngNo; i++) {
    bool inputState = digitalRead(chng[i]);
    if (inputState != chngPrev[i]) {
      chngDebounce[i]++;
    }
    if (chngDebounce[i] >= debounce) {
      if(debug) Serial.print(chngName[i]);
      if(debug) Serial.println(" - CHANGE");
      Keyboard.print(chngChar[i]);
      chngPrev[i] = !chngPrev[i];
      chngDebounce[i] = 0;
    }
  }

  delay(2);
}
